dnl eb.m4 --- Automake macros for working with EB library.
dnl Copyright (C) 1999 NISHIDA Keisuke <knishida@ring.aist.go.jp>
dnl  
dnl This program is free software; you can redistribute it and/or modify
dnl it under the terms of the GNU General Public License as published by
dnl the Free Software Foundation; either version 2, or (at your option)
dnl any later version.
dnl 
dnl This program is distributed in the hope that it will be useful,
dnl but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl GNU General Public License for more details.
dnl 
dnl You should have received a copy of the GNU General Public License
dnl along with this software; see the file COPYING.  If not, write to
dnl the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
dnl Boston, MA 02111-1307 USA
dnl
dnl Modified by SUZUKI, Mitsuhiro<hige@higesoft.rim.or.jp>
dnl 
AC_DEFUN([EB_CONFIG],[
  dnl
  dnl check for EB library
  dnl
  AC_ARG_WITH(eb-includes,
    [  --with-eb-includes=DIR    EB include files are in DIR],
    EB_INCLUDES=$withval, EB_INCLUDES=$includedir)

  AC_ARG_WITH(eb-libraries,
    [  --with-eb-libraries=DIR   EB library files are in DIR],
    EB_LIBRARIES=$withval, EB_LIBRARIES=$libdir)
  AC_ARG_WITH(zlib-includes,
    [  --with-zlib-includes=DIR  zlib include files are in DIR],
    ZLIB_INCLUDES=$withval, ZLIB_INCLUDES=$includedir)

  AC_ARG_WITH(zlib-libraries,
    [  --with-zlib-libraries=DIR zlib library files are in DIR],
    ZLIB_LIBRARIES=$withval, ZLIB_LIBRARIES=$libdir)

  EB_INCLUDES="-I$EB_INCLUDES"
  if test $EB_INCLUDES != $ZLIB_INCLUDES; then
    EB_INCLUDES="$EB_INCLUDES -I$ZLIB_INCLUDES"
  fi
  EB_LDADD="-L$EB_LIBRARIES"
  if test $EB_LIBRARIES != $ZLIB_LIBRARIES; then
    EB_LDADD="$EB_LDADD -L$ZLIB_LIBRARIES"
  fi
  EB_LDADD="$EB_LDADD -leb -lz"
])
AC_DEFUN([EB_CHECK_LIB], [
  AC_MSG_CHECKING(for libeb)
  AC_REQUIRE([EB_CONFIG])
  AC_CACHE_VAL(ac_cv_eb_ldadd,[
    ac_save_CC="$CC"
    CC="$CC $EB_INCLUDES"
    AC_LANG_SAVE
    AC_LANG_C
    ac_save_LIBS="$LIBS"
    LIBS="$LIBS $EB_LDADD"
    AC_TRY_LINK(
      [
        #ifdef __cplusplus
        extern "C" {
        #endif
          int eb_canonicalize_filename (char *);
        #ifdef __cplusplus
        }
        #endif
      ], [
        char filename[] = ""; eb_canonicalize_filename(filename);
      ],
      eb_ok=yes,
      eb_ok=no
      AC_MSG_ERROR(*** EB library not found ***))
    AC_LANG_RESTORE
    CC="$ac_save_CC"
    LIBS="$ac_save_LIBS"
    ac_cv_eb_ldadd="$EB_LDADD"
  ])
  AC_MSG_RESULT("-leb -lz")
  AC_SUBST(EB_LDADD)
])
AC_DEFUN([EB_CHECK_HEADER], [
  AC_REQUIRE([EB_CONFIG])
  AC_MSG_CHECKING([for eb/eb.h])
  AC_CACHE_VAL(ac_cv_eb_includes, [
    ac_save_CPP="$CPP"
    CPP="$CPP $EB_INCLUDES"
    AC_LANG_SAVE
    AC_LANG_C
    AC_TRY_CPP([
      #include <stdio.h>
      #undef PACKAGE
      #undef VERSION
      #include <eb/eb.h>],
      eb_ok=yes,
      eb_ok=no)
    AC_LANG_RESTORE
    if test "$eb_ok" = no; then
      AC_MSG_RESULT(no)
      AC_MSG_ERROR(*** EB header file not found ***)
    fi
    CPP="$ac_save_CPP"
    ac_cv_eb_cflags="$EB_INCLUDES"
  ]) 
  AC_MSG_RESULT(yes)
  AC_SUBST(EB_INCLUDES)
])
AC_DEFUN([EB_CHECK], [
  AC_REQUIRE([EB_CHECK_LIB])
  AC_REQUIRE([EB_CHECK_HEADER])
])
