/*
 * $Id: cebook.cpp,v 1.3 2001/01/13 19:46:35 hige Exp $
 *
 * Copyright (C) 1999 SUZUKI, Mitsuhiro <hige@higesoft.rim.or.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include "cebook.hpp"

using namespace ebook;

static string* g_font_path;

// Constants
static const int BUFSIZE                = 4096;
static const int MAX_HITS               = 64;
static const int BASE_FONT_SIZE         = 16;
static const int SHOW_PROGRESS_COUNT    = 4;
static const char INSERT_MARK           = '\a';
static const char DIRECTORY_SEPARATOR   = '/';

#define EMPTY                             0

// Hook
#define HOOK_PROTOTYPE(hook)    int hook (EB_Book* book,  EB_Appendix* appendix,  \
                                          char* buff,  EB_Hook_Code code,  \
                                          int argc, const int *argv)

HOOK_PROTOTYPE(hook_font);
HOOK_PROTOTYPE(hook_newline);
HOOK_PROTOTYPE(hook_tags);

HOOK_PROTOTYPE(heading_hook_font);
HOOK_PROTOTYPE(heading_hook_newline);
HOOK_PROTOTYPE(heading_hook_tags);

// Hook Table
static EB_Hook text_hooks[] = {
    {EB_HOOK_NARROW_JISX0208, eb_hook_euc_to_ascii},
    {EB_HOOK_NARROW_FONT,     hook_font},
    {EB_HOOK_WIDE_FONT,       hook_font},
    {EB_HOOK_NEWLINE,         hook_newline},
    {EB_HOOK_STOPCODE,        eb_hook_stopcode_mixed},
    {EB_HOOK_BEGIN_PICTURE,   hook_tags},
    {EB_HOOK_END_PICTURE,     hook_tags},
    {EB_HOOK_BEGIN_MENU,      hook_tags},
    {EB_HOOK_END_MENU,        hook_tags},
    {EB_HOOK_BEGIN_SOUND,     hook_tags},
    {EB_HOOK_END_SOUND,       hook_tags},
    {EB_HOOK_BEGIN_REFERENCE, hook_tags},
    {EB_HOOK_END_REFERENCE,   hook_tags},
    {EB_HOOK_NULL, EMPTY},
};
static EB_Hook heading_text_hooks[] = {
    {EB_HOOK_NARROW_JISX0208, eb_hook_euc_to_ascii},
    {EB_HOOK_NARROW_FONT,     heading_hook_font},
    {EB_HOOK_WIDE_FONT,       heading_hook_font},
    {EB_HOOK_NEWLINE,         heading_hook_newline},
    {EB_HOOK_STOPCODE,        eb_hook_stopcode_mixed},
    {EB_HOOK_BEGIN_PICTURE,   heading_hook_tags},
    {EB_HOOK_END_PICTURE,     heading_hook_tags},
    {EB_HOOK_BEGIN_MENU,      heading_hook_tags},
    {EB_HOOK_END_MENU,        heading_hook_tags},
    {EB_HOOK_BEGIN_SOUND,     heading_hook_tags},
    {EB_HOOK_END_SOUND,       heading_hook_tags},
    {EB_HOOK_BEGIN_REFERENCE, heading_hook_tags},
    {EB_HOOK_END_REFERENCE,   heading_hook_tags},
    {EB_HOOK_NULL, EMPTY},
};

static int g_font_size = 16;

static vector<EB_Position> pos_array;

EB_Font_Code                g_fontlist[EB_MAX_FONTS];

#define  HTML_TABLE_START               "<table cellspacing=\"0\" cellpadding=\"0\"><tr>"
#define  HTML_TABLE_END                 "</tr></table>"
#define  HTML_TABLE_DATA_START          "<td nowrap>"
#define  HTML_TABLE_DATA_END            "</td>"
#define  HTML_IMG                       "<img "

const size_t HTML_TABLE_START_LEN       = strlen(HTML_TABLE_START);
const size_t HTML_TABLE_END_LEN         = strlen(HTML_TABLE_END);
const size_t HTML_TABLE_DATA_START_LEN  = strlen(HTML_TABLE_DATA_START);
const size_t HTML_TABLE_DATA_END_LEN    = strlen(HTML_TABLE_DATA_END);
const size_t HTML_IMG_LEN               = strlen(HTML_IMG);

bool save_font_image(const char* filename, const char* imagedata, size_t imagesize)
{
    FILE *fp = fopen(filename, "wb");

    if (fp == EMPTY) {
        cerr << "Cannot open the font file, "
             << strerror(errno) << filename
             << endl;
        return false;
    }
    if (fwrite(imagedata, 1, imagesize, fp) < imagesize) {
        cerr << "Error fwrite :" << filename << endl;
        return false;
    }

    fclose(fp);

    return true;
}
void fontpath(string& fontfilename, const char* fonttype, int c)
{
    char filetitle[4+1];
    sprintf(filetitle, "%04x", c);
    fontfilename = *g_font_path+DIRECTORY_SEPARATOR;
    fontfilename += fonttype;
    fontfilename += DIRECTORY_SEPARATOR;
    fontfilename += string(filetitle);
    fontfilename += ".xbm";
}
bool isfile(const string& filename)
{
    struct stat sb;
    if (stat(filename.c_str(), &sb) != RC_ERR &&
        S_ISREG(sb.st_mode)){
        return true;
    }
    return false;
}
bool createNarrowFontFile(EB_Book* book, const string& fontfilename, int ch)
{
    char bitmap[EB_SIZE_NARROW_FONT_48];
    EB_Font_Code *list = g_fontlist;
    while (!eb_have_font(book, *list)) {
        list++;
        if (list-g_fontlist >= EB_MAX_FONTS) {
            return false;
        }
    }
    if (eb_set_font(book, BASE_FONT_SIZE) < 0) {
        cerr << eb_error_message() << endl;
        return false;
    }
    if (eb_narrow_font_character_bitmap(book, ch, bitmap) < 0) {
        cerr << eb_error_message() << endl;
        return false;
    }
                
    char imagedata[EB_SIZE_FONT_IMAGE];
    size_t imagelength = eb_bitmap_to_xbm(imagedata, bitmap,
                                          BASE_FONT_SIZE/2, BASE_FONT_SIZE);
    eb_unset_font(book);
    save_font_image(fontfilename.c_str(), imagedata, imagelength);
    return true;
}
bool createWideFontFile(EB_Book* book, const string& fontfilename, int ch)
{
    char bitmap[EB_SIZE_WIDE_FONT_48];

    EB_Font_Code *list = g_fontlist;
    while (!eb_have_font(book, *list)) {
        list++;
        if (list-g_fontlist >= EB_MAX_FONTS) {
            return false;
        }
    }
    eb_set_font(book, BASE_FONT_SIZE);
    if (eb_wide_font_character_bitmap(book, ch, bitmap) < 0) {
        cerr << eb_error_message() << endl;
        return false;
    }
    char imagedata[EB_SIZE_FONT_IMAGE];
    size_t imagelength = eb_bitmap_to_xbm(imagedata, bitmap,
                                          BASE_FONT_SIZE, BASE_FONT_SIZE);
    eb_unset_font(book);
    
    save_font_image(fontfilename.c_str(), imagedata, imagelength);

    return true;
}
int hook_font(EB_Book* book,
              EB_Appendix* appendix,
              char* buff,
              EB_Hook_Code code,
              int, const int *argv)
{
    switch (code) {
    case EB_HOOK_NARROW_FONT:
        if (eb_narrow_alt_character_text(appendix, argv[0], buff) < 0) {
            string fontfilename;
            
            fontpath(fontfilename, "narrow", argv[0]);
            sprintf(buff,
                    HTML_TABLE_DATA_END HTML_TABLE_DATA_START
                    HTML_IMG "src=\"file:%s\" alt=\"GAIJI=%04x\" "
                    "width=\"%d\" height=\"%d\">"
                    HTML_TABLE_DATA_END HTML_TABLE_DATA_START,
                    fontfilename.c_str(), argv[0],
                    g_font_size/2, g_font_size);
            if (!isfile(fontfilename)) {
                createNarrowFontFile(book, fontfilename, argv[0]);
            }
        }
        break;
    case EB_HOOK_WIDE_FONT:
        if (eb_wide_alt_character_text(appendix, argv[0], buff) < 0) {
            string fontfilename;

            fontpath(fontfilename, "wide", argv[0]);
            sprintf(buff,
                    HTML_TABLE_DATA_END HTML_TABLE_DATA_START
                    HTML_IMG "src=\"file:%s\" alt=\"GAIJI=%04x\" "
                    "width=\"%d\" height=\"%d\">"
                    HTML_TABLE_DATA_END HTML_TABLE_DATA_START,
                    fontfilename.c_str(), argv[0],
                    g_font_size, g_font_size);
            if (!isfile(fontfilename)) {
                createWideFontFile(book, fontfilename, argv[0]);
            }
        }
        break;
    }
    return 0;
}
int heading_hook_font(EB_Book*,
                      EB_Appendix* appendix,
                      char* buff,
                      EB_Hook_Code code,
                      int , const int *argv)
{
    switch (code) {
    case EB_HOOK_NARROW_FONT:
        if (eb_narrow_alt_character_text(appendix, argv[0], buff) < 0) {
            strcpy(buff, "?");
        }
        break;
    case EB_HOOK_WIDE_FONT:
        if (eb_wide_alt_character_text(appendix, argv[0], buff) < 0) {
            strcpy(buff, "??");
        }
        break;
    }
    return 0;
}

int hook_newline(EB_Book*,
                 EB_Appendix*,
                 char* buff,
                 EB_Hook_Code,
                 int, const int*)
{
    strcpy(buff,
           HTML_TABLE_DATA_END
           HTML_TABLE_END
           HTML_TABLE_START
           HTML_TABLE_DATA_START);
    return 0;
}
int heading_hook_newline(EB_Book*,
                 EB_Appendix*,
                 char* buff,
                 EB_Hook_Code,
                 int, const int*)
{
    strcpy(buff, "\n");
    return 0;
}
int hook_tags(EB_Book*,
              EB_Appendix*,
              char* buff,
              EB_Hook_Code code,
              int, const int* argv)
{
    EB_Position pos;
    switch (code) {
    case EB_HOOK_BEGIN_PICTURE:
        sprintf(buff, "<a href=\"ebpict://%c\">", INSERT_MARK);
        break;
    case EB_HOOK_END_PICTURE:
        strcpy(buff, "</a>");
        pos.page = argv[1];
        pos.offset = argv[2];
        pos_array.push_back(pos);
        break;

    case EB_HOOK_BEGIN_SOUND:
        sprintf(buff, "<a href=\"ebsound://%c\">", INSERT_MARK);
        break;
    case EB_HOOK_END_SOUND:
        strcpy(buff, "</a>");
        pos.page = argv[1];
        pos.offset = argv[2];
        pos_array.push_back(pos);
        break;

    case EB_HOOK_BEGIN_MENU:
    case EB_HOOK_BEGIN_REFERENCE:
        sprintf(buff, "<a href=\"eb://%c\">", INSERT_MARK);
        break;
    case EB_HOOK_END_MENU:
    case EB_HOOK_END_REFERENCE:
        strcpy(buff, "</a>");
        pos.page = argv[1];
        pos.offset = argv[2];
        pos_array.push_back(pos);
        break;
    }
    return 0;
}
int heading_hook_tags(EB_Book*,
                      EB_Appendix*,
                      char*,
                      EB_Hook_Code,
                      int, const int*)
{
    return 0;
}
CEBook::CEBook()
    : m_subbooks(EMPTY), m_wrapcol(40)
{
    m_font_path = new string("");
    //m_subbook_directory = new string("");
    
    eb_initialize(&m_book);
    eb_initialize_appendix(&m_appendix);
    
    eb_initialize_hookset(&m_text_hookset);
    eb_set_hooks(&m_text_hookset, text_hooks);

    eb_initialize_hookset(&m_heading_text_hookset);
    eb_set_hooks(&m_heading_text_hookset, heading_text_hooks);
}
CEBook::CEBook(const char* dict_path, const char* font_path)
{
    setDictPath(dict_path);
    setFontPath(font_path);
}
CEBook::~CEBook()
{
    delete m_subbooks;
    delete m_font_path;
    //delete m_subbook_directory;
    eb_clear(&m_book);
}
void CEBook::setSubbookInfoList()
{
    EB_Subbook_Code sub_books[EB_MAX_SUBBOOKS];
    int count = eb_subbook_list(&m_book, sub_books);
    if (count == RC_ERR) {
        m_error.setCurrentError();
        return;
    }
    m_subbooks = new SubbookInfoList();
    int i = 0;
    while (i < count) {
        SubbookInfo info;
        info.title = eb_subbook_title2(&m_book, sub_books[i]);
        info.directory = eb_subbook_directory2(&m_book, sub_books[i]);
        info.code = sub_books[i];
        m_subbooks->push_back(info);
        i++;
    }
    m_cur_subbook = m_subbooks->begin();
}
int CEBook::getSubbook()
{
    if (m_subbooks != EMPTY
        && !m_subbooks->empty()
        && m_cur_subbook != m_subbooks->end())
        return m_cur_subbook-m_subbooks->begin();
    else
        return RC_ERR;
}
void CEBook::setSubbook(SubbookInfoListIterator it)
{
    if (eb_set_subbook(&m_book, (*it).code) == RC_ERR) {
        m_error.setCurrentError();
        return;
    }
    m_cur_subbook = it;
    eb_font_list(&m_book, g_fontlist);
}
const char* CEBook::getSubbookDirectory()
{
    return (*m_cur_subbook).directory.c_str();
}
void CEBook::getHeading(string& heading, EB_Position* pos)
{
    heading = "";
    char buff[BUFSIZE];
    buff[sizeof (buff)-1] = '\0';

    eb_seek(&m_book, pos);
    
    g_font_path = m_font_path;
    while (eb_text(&m_book, &m_appendix,
                 &m_heading_text_hookset,
                 buff, sizeof (buff)-1) > 0) {
        char* nl;
        if ((nl = strchr(buff, '\n')) != EMPTY) {
            *nl = '\0';
            heading += buff;
            break;
        } else {
            heading += buff;
        }
    }
}
void CEBook::getContent(string& content, EB_Position* pos)
{
    content = HTML_TABLE_START HTML_TABLE_DATA_START;
    
    char buff[BUFSIZE];
    buff[sizeof (buff)-1] = '\0';
    string body = "";
    
    eb_seek(&m_book, pos);
    g_font_path = m_font_path;
    while (eb_text(&m_book, &m_appendix, &m_text_hookset,
                   buff, sizeof (buff)-1) > 0) {
        body += buff;
    }
    // Insert Link Address
    vector<EB_Position>::iterator it = pos_array.begin();
    string::iterator p = body.begin();
    while (p != body.end()) {
        if (*p != INSERT_MARK) {
            content += *p;
        } else {
            char page_offset[32];
            sprintf(page_offset, "%d:%d", (*it).page, (*it).offset);
            content += page_offset;
            it++;
        }
        p++;
    }
    pos_array.clear();
    
    content += HTML_TABLE_DATA_END HTML_TABLE_END;
}
void CEBook::getContent(string& content, int page, int offset)
{
    EB_Position pos;
    pos.page = (int)page;
    pos.offset = (int)offset;
    getContent(content, &pos);
}

struct less_EB_Position :
    binary_function <EB_Position, EB_Position, bool>
{
    bool operator()(const EB_Position& a, const EB_Position& b) const
    {
        if (a.page != b.page) {
            return a.page < b.page;
        } else {
            return a.offset < b.offset;
        }
    }
};
void CEBook::lookupWord(CEBook::LookupClient* client,
                        CEBook::LookupResult& result,
                        const char* word,
                        CEBook::WAY_TO_SEARCH way)
{
    int (*have_search)(EB_Book*);
    int (*search_word)(EB_Book*, const char*);
    switch (way) {
    case EXACT:
        have_search = eb_have_exactword_search;
        search_word = eb_search_exactword;
        break;
    case FRONT:
        have_search = eb_have_word_search;
        search_word = eb_search_word;
        break;
    case TAIL:
        have_search = eb_have_endword_search;
        search_word = eb_search_endword;
        break;
    default:
        have_search = eb_have_exactword_search;
        search_word = eb_search_exactword;
        break;
    }
    if (!(*have_search)(&m_book)
        || (*search_word)(&m_book, word) == RC_ERR) {
        m_error.setCurrentError();
        return;
    }
    
    EB_Hit hits[MAX_HITS];
    map<EB_Position, string, less_EB_Position> prevresult;
    int hitcount, totalcount = 0;
    string heading, content;
    int counter_showprogress = 1;
    
    while ((hitcount = eb_hit_list(&m_book, hits, MAX_HITS)) > 0) {
        for (int i = 0; i < hitcount; i++) {
            getHeading(heading, &hits[i].heading);
            // Ignore a hit entry which equals to
            // a previous hit entry.
            if (!prevresult.empty() &&
                prevresult.count(hits[i].text) &&
                prevresult[hits[i].text] == heading) {
                continue;
            }
            getContent(content, &hits[i].text);
            client->enterCriticalSection(&result);
            result.push_back(LookupResultElement(heading, content));
            client->leaveCriticalSection(&result);
            if (client != EMPTY) {
                if (counter_showprogress >= SHOW_PROGRESS_COUNT
                    || i == hitcount-1) {
                    client->showProgress(totalcount-counter_showprogress+1,
                                         counter_showprogress,
                                         (i*100)/hitcount);
                    counter_showprogress = 0;
                }
            }
            prevresult[hits[i].text] = heading;
            
            if (client->isStopped()) {
                if (counter_showprogress != 0) {
                    client->showProgress(totalcount-counter_showprogress+1,
                                         counter_showprogress,
                                         (i*100)/hitcount);
                }
                return;
            }
            totalcount++;
            counter_showprogress++;
        }
    }
}
void CEBook::getCopyright(string& copyright)
{
    EB_Position pos;
    if (eb_copyright(&m_book, &pos) == RC_ERR) {
        copyright = "This dictionry has no copyright information.";
    } else {
        getContent(copyright, pos.page, pos.offset);
    }
    
}
void CEBook::getDictMenu(string& menu)
{
    EB_Position pos;
    if (eb_have_menu(&m_book)) {
        eb_menu(&m_book, &pos);
        getContent(menu, pos.page, pos.offset);
    } else {
        menu = "This dictinary has no menu.";
    }
}
void CEBook::setDictPath(const char* dict_path)
{
    if (eb_bind(&m_book, dict_path) == RC_ERR) {
        m_error.setCurrentError();
    }
    setSubbookInfoList();
}
void CEBook::setFontPath(const char* font_path)
{
    *m_font_path = font_path;
}
const char* CEBook::getFontPath()
{
    return m_font_path->c_str();
}
int CEBook::getFontSize()
{
    return g_font_size;
}
void CEBook::setFontSize(int font_size)
{
    g_font_size = font_size;
}
bool CEBook::isAvailableDictPath(const char* path)
{
    EB_Book book;
    eb_initialize(&book);
    if (eb_bind(&book, path) == RC_ERR) {
        eb_clear(&book);
        return false;
    }
    eb_clear(&book);
    return true;
}
const char*         CEBook::getRowStart()
{
    return HTML_TABLE_START HTML_TABLE_DATA_START;
}
const char*         CEBook::getRowEnd()
{
    return HTML_TABLE_DATA_END HTML_TABLE_END;
}
