/*
 * $Id: changecolordlg.cpp,v 1.1.1.1 2000/05/25 06:09:58 hige Exp $
 *
 * Copyright (C) 1999 SUZUKI, Mitsuhiro <hige@higesoft.rim.or.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 */
#include <kcolordlg.h>
#include <kcolorbtn.h>
#include <klocale.h>

#include <qlayout.h>
#include <qlabel.h>
#include "changecolordlg.h"
#include "htmlcolors.h"

ChangeColorDlg::ChangeColorDlg(const KResultPart::ColorAttribute& colorattr,
                               QWidget *parent, const char *name,
                               WFlags f)
    : KDialog(parent, name, true, f),
      m_colorattr(new KResultPart::ColorAttribute(colorattr))
{
    QVBoxLayout* main_layout = new QVBoxLayout(this);
    CHECK_PTR(main_layout);
    QGridLayout* color_button_layout = new QGridLayout(3, 2);
    CHECK_PTR(color_button_layout);
    main_layout->addLayout(color_button_layout);
    
    QLabel* bgcolorlabel = new QLabel(i18n("Background Color"), this);
    CHECK_PTR(bgcolorlabel);
    color_button_layout->addWidget(bgcolorlabel, 0, 0);
    
    m_btnbackgroundcolor = new KColorButton(colorattr.background, this);
    CHECK_PTR(m_btnbackgroundcolor);
    connect(m_btnbackgroundcolor, SIGNAL(changed(const QColor&)),
            SLOT(setBackgroundColor(const QColor&)));
    color_button_layout->addWidget(m_btnbackgroundcolor, 0, 1);
    
    QLabel* textcolorlabel = new QLabel(i18n("Text Color"), this);
    CHECK_PTR(textcolorlabel);
    color_button_layout->addWidget(textcolorlabel, 1, 0);

    m_btntextcolor = new KColorButton(colorattr.text, this);
    CHECK_PTR(m_btntextcolor);
    connect(m_btntextcolor, SIGNAL(changed(const QColor&)),
            SLOT(setTextColor(const QColor&)));
    color_button_layout->addWidget(m_btntextcolor, 1, 1);

    QLabel* linkcolorlabel = new QLabel(i18n("Link Color"), this);
    CHECK_PTR(linkcolorlabel);
    color_button_layout->addWidget(linkcolorlabel, 2, 0);

    m_btnlinkcolor = new KColorButton(colorattr.link, this);
    CHECK_PTR(m_btnlinkcolor);
    connect(m_btnlinkcolor, SIGNAL(changed(const QColor&)),
            SLOT(setLinkColor(const QColor&)));
    color_button_layout->addWidget(m_btnlinkcolor, 2, 1);
    
    QHBoxLayout* button_layout = new QHBoxLayout();
    CHECK_PTR(button_layout);
    main_layout->addLayout(button_layout);
    QPushButton* changedefault = new QPushButton(i18n("Default"), this);
    CHECK_PTR(changedefault);
    connect(changedefault, SIGNAL(clicked()), SLOT(setReturnDefault()));
    button_layout->addWidget(changedefault);
    
    QPushButton* ok = new QPushButton(i18n("OK"), this);
    CHECK_PTR(ok);
    connect(ok, SIGNAL(clicked()), SLOT(accept()));
    button_layout->addWidget(ok);

    QPushButton* cancel = new QPushButton(i18n("Cancel"), this);
    CHECK_PTR(cancel);
    connect(cancel, SIGNAL(clicked()), SLOT(reject()));
    button_layout->addWidget(cancel);

    main_layout->activate();
    main_layout->freeze();
}
ChangeColorDlg::~ChangeColorDlg()
{
    delete m_colorattr;
}
void ChangeColorDlg::setReturnDefault()
{
    m_btnbackgroundcolor->setColor(HTML_DEFAULT_BACKGROUND_COLOR);
    setBackgroundColor(HTML_DEFAULT_BACKGROUND_COLOR);
    m_btntextcolor->setColor(HTML_DEFAULT_TEXT_COLOR);
    setTextColor(HTML_DEFAULT_TEXT_COLOR);
    m_btnlinkcolor->setColor(HTML_DEFAULT_LINK_COLOR);
    setLinkColor(HTML_DEFAULT_LINK_COLOR);
}
