/*
 * $Id: kebook.cpp,v 1.9 2001/01/31 07:13:19 hige Exp $
 *
 * Copyright (C) 1999 SUZUKI, Mitsuhiro <hige@higesoft.rim.or.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <qkeycode.h>

#include <kapp.h>
#include <kiconloader.h>
#include <kfiledialog.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kstatusbar.h>
#include <kmenubar.h>
#include <kpopupmenu.h>
#include <kaccel.h>
#include <kthemestyle.h>

#include "kebook.h"
#include "kebookwidget.h"
#include "selectsubbookdlg.h"
#include "selectwaytosearchdlg.h"
#include "htmlcolors.h"

//
// KEBook class member functions (private)
//

void KEBook::updateDictDirHistoryMenu()
{
    m_dictdir_popup->clear();
    
    KEBookApp::DictDirHistory::const_iterator it;
    int id;
    for (it = getApp()->getBeginOfDictDirHistory(), id = 0;
         it != getApp()->getEndOfDictDirHistory();
         it++, id++) {
        m_dictdir_popup->insertItem(QString::fromLocal8Bit((*it).c_str()), id);
    }
    
}
void KEBook::createFileMenu()
{
    QPopupMenu* file = new QPopupMenu();
    m_dictdir_popup = new QPopupMenu;
    CHECK_PTR(m_dictdir_popup);
    
    updateDictDirHistoryMenu();
    
    file->insertItem(BarIcon(OPENDIR_TBBTN_PIXMAP),
                     i18n("Recent Dictionary Directories"), m_dictdir_popup);
    
    this->connect(m_dictdir_popup,
                  SIGNAL(activated(int)),
                  SLOT(onSelectDictDirFromList(int)));
    
    file->insertItem(BarIcon(EXIT_TBBTN_PIXMAP),
                     i18n("&Quit"), this, SLOT(quitKEBook()),
                     CTRL+Key_Q, ID_MENUITEM_QUIT);
    
    menuBar()->insertItem(i18n("&File"), file, (int)ID_MENU_FILE);
}
void KEBook::createEditMenu()
{
    QPopupMenu* edit = new QPopupMenu();
    edit->insertItem(i18n("&Find"),
                     view, SLOT(slotFindWordInPage()),
                     CTRL+SHIFT+Key_F2,
                     ID_MENUITEM_FIND);
    edit->insertItem(i18n("Find &Next"),
                     view, SLOT(slotFindNextWordInPage()),
                     CTRL+SHIFT+Key_F3,
                     ID_MENUITEM_FINDNEXT);
    menuBar()->insertItem(i18n("&Edit"), edit, (int)ID_MENU_EDIT);
}
void KEBook::createShowMenu()
{
    QPopupMenu* show = new QPopupMenu();
    show->insertItem(BarIcon(MENU_TBBTN_PIXMAP), i18n("&Menu"),
                     this, SLOT(showDictMenu()), 0,
                     ID_MENUITEM_DICT_MENU);
    show->setItemEnabled(ID_MENUITEM_DICT_MENU, false);
    
    show->insertItem(BarIcon(COPYRIGHT_TBBTN_PIXMAP), i18n("&Copyright"),
                     this, SLOT(showDictCopyright()), 0,
                     ID_MENUITEM_COPYRIGHT);
    show->setItemEnabled(ID_MENUITEM_COPYRIGHT, false);
    menuBar()->insertItem(i18n("&Show"), show, (int)ID_MENU_SHOW);
}
void KEBook::createGoMenu()
{
    QPopupMenu* go = new QPopupMenu();
    go->insertItem(BarIcon(BACK_TBBTN_PIXMAP),
                   i18n("&Back"), view, SLOT(onBack()),
                   CTRL+Key_P, ID_MENUITEM_BACK);
    go->setItemEnabled(ID_MENUITEM_BACK, false);
    
    go->insertItem(BarIcon(FORWARD_TBBTN_PIXMAP),
                   i18n("&Forward"), view, SLOT(onForward()),
                   CTRL+Key_N, ID_MENUITEM_FORWARD);
    go->setItemEnabled(ID_MENUITEM_FORWARD, false);
    menuBar()->insertItem(i18n("&Go"), go, (int)ID_MENU_GO);
}
void KEBook::createOptionMenu()
{
    QPopupMenu* setup = new QPopupMenu;
    setup->insertItem(BarIcon(WAY_TO_SEARCH_TBBTN_PIXMAP),
                      i18n("Select a &Way to search..."),
                      this, SLOT(openSelectWayToSearchDlg()),
                      CTRL+SHIFT+Key_W,
                      ID_MENUITEM_OPEN_SELECT_WAY);

    setup->insertItem(BarIcon(SETSUBBOOK_TBBTN_PIXMAP),
                      i18n("&Select a Dictionary..."),
                      this, SLOT(openSelectSubbookDlg()),
                      CTRL+SHIFT+Key_S,
                      ID_MENUITEM_OPEN_SELECT_DICT_DLG);
    setup->setItemEnabled(ID_MENUITEM_OPEN_SELECT_DICT_DLG, false);

    setup->insertItem(BarIcon(OPENDIR_TBBTN_PIXMAP),
                      i18n("Set a &Directory to contain the dictionary..."),
                      this, SLOT(openDictPathDlg()),
                      CTRL+SHIFT+Key_D,
                      ID_MENUITEM_OPEN_SET_DIR_DLG);

    setup->insertItem(BarIcon(KEYBOARD_TBBTN_PIXMAP),
                      i18n("Edit Keyboard accelerators..."),
                      view, SLOT(openKeyDlg()),
                      CTRL+SHIFT+Key_A,
                      ID_MENUITEM_OPEN_EDIT_ACCEL_DLG);
    setup->insertItem(i18n("Change Color..."),
                      view, SLOT(openColorDlg()),
                      0, ID_MENUITEM_OPEN_COLOR_DLG);
    menuBar()->insertItem(i18n("&Options"), setup, (int)ID_MENU_OPTION);
}
void KEBook::createCDROMMenu()
{
    if (rescanDictionariesInCDROM()) {
        QPopupMenu* cdrom = new QPopupMenu();
        cdrom->insertItem(BarIcon(DICTIONARIES_TBBTN_PIXMAP),
                          i18n("Select dictionary"),
                          m_dictionary_list_popup);
        this->connect(m_dictionary_list_popup,
                      SIGNAL(activated(int)),
                      SLOT(onSelectDictionaryFromList(int)));
        this->connect(m_function_key,
                      SIGNAL(activated(int)),
                      SLOT(onSelectDictionaryFromList(int)));
        cdrom->insertSeparator();
        cdrom->insertItem(BarIcon(RELOAD_TBBTN_PIXMAP),
                          i18n("Rescan dictionaries"), this,
                          SLOT(rescanDictionariesInCDROM()),
                          CTRL+SHIFT+Key_R, ID_MENUITEM_RESCAN_DICTIONARIES);
        menuBar()->insertItem(i18n("&CD-ROM"), cdrom, ID_MENU_CDROM);
    } else {
        menuBar()->insertItem(i18n("&CD-ROM"), ID_MENU_CDROM);
        menuBar()->setItemEnabled(ID_MENU_CDROM, false);
    }
}
void KEBook::createHelpMenu()
{
    // KDE will generate a short help menu automagically
    QString about_message = "KEBook2 --- Version " VERSION "\n\n";
    
    QPopupMenu* help =
        helpMenu(about_message+
                 i18n("Copyright (C) 1999 SUZUKI, Mitsuhiro <hige@higesoft.rim.or.jp>\n" 
                      "KEBook is an application with which you look up word \n" 
                      "in your CD-ROM books of EB, EBG, EBXA, EBXA-C, S-EBXA and EPWING formats \n" 
                      " \n" 
                      "This program is free software; you can redistribute it and/or modify \n" 
                      "it under the terms of the GNU General Public License as published by \n" 
                      "the Free Software Foundation; either version 2, or (at your option) \n" 
                      "any later version. \n" 
                      " \n" 
                      "This program is distributed in the hope that it will be useful, \n" 
                      "but WITHOUT ANY WARRANTY; without even the implied warranty of \n" 
                      "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the \n" 
                      "GNU General Public License for more details.\n"));
    menuBar()->insertItem(i18n("&Help"), help, ID_MENU_HELP);
}
void KEBook::createMainMenu()
{
    // Menu ``File''
    createFileMenu();
    
    // Menu ``Edit''
    createEditMenu();
    
    // Menu ``Show''
    createShowMenu();
    
    // Menu ``Go''
    createGoMenu();
    
    // Menu ``Option''
    createOptionMenu();
    
    // Menu ``CD-ROM''
    createCDROMMenu();
    
    menuBar()->insertSeparator();
    
    // Menu ``HELP''
    createHelpMenu();
}
void KEBook::createWayToSearchPopupMenu()
{
    QPopupMenu* way_to_search_popup = new QPopupMenu;
    KAccel* accel = new KAccel(this);
    vector<WayToSearchTableElement>::iterator it;
    it = m_way_to_search_table->begin();
    while (it != m_way_to_search_table->end()) {
        QString descript = it->getDescription();
        descript += "\t";
        descript += KAccel::keyToString(it->getAccel());
        way_to_search_popup->insertItem(BarIcon(it->getIconFile()), descript,
                                        &(*it), SLOT(setWayToSearch()));
        accel->insertItem(it->getDescription(), it->getAction(), it->getAccel());
        accel->connectItem(it->getAction(), &(*it), SLOT(setWayToSearch()));
        ++it;
    }

    toolBar()->insertButton(BarIcon(WAY_TO_SEARCH_TBBTN_PIXMAP),
                            ID_TBBTN_WAY_TO_SEARCH,
                            way_to_search_popup, true, i18n("Choose a way to search"));
    updateSelectionOfWayToSearch();

}
void KEBook::createToolBar()
{
    // insert two buttons "Forward & Back"
    toolBar()->insertButton(BarIcon(BACK_TBBTN_PIXMAP),
                            ID_TBBTN_BACK, SIGNAL(clicked()),
                            view, SLOT(onBack()), false, 
                            i18n("Back history"));
    
    toolBar()->insertButton(BarIcon(FORWARD_TBBTN_PIXMAP),
                            ID_TBBTN_FORWARD, SIGNAL(clicked()),
                            view, SLOT(onForward()), false, 
                            i18n("Forward history"));

    toolBar()->insertButton(BarIcon(STOP_TBBTN_PIXMAP),
                            ID_TBBTN_STOPLOOKINGUP, SIGNAL(clicked()),
                            kapp, SLOT(stopLookingUp()), false,
                            i18n("Stop looking up"));
    
    toolBar()->insertButton(BarIcon(SETSUBBOOK_TBBTN_PIXMAP),
                            ID_TBBTN_SETSUBBOOK, SIGNAL(clicked()),
                            this, SLOT(openSelectSubbookDlg()), false,
                            i18n("Select a Dictionary"));

    toolBar()->insertButton(BarIcon(OPENDIR_TBBTN_PIXMAP),
                            ID_TBBTN_OPENDIR, SIGNAL(clicked()),
                            this, SLOT(openDictPathDlg()), true,
                            i18n("Set a directory to contain the dictionary"));

    toolBar()->insertButton(BarIcon(MENU_TBBTN_PIXMAP),
                            ID_TBBTN_MENU, SIGNAL(clicked()),
                            this, SLOT(showDictMenu()), false,
                            i18n("Menu"));
    toolBar()->insertButton(BarIcon(COPYRIGHT_TBBTN_PIXMAP),
                            ID_TBBTN_COPYRIGHT, SIGNAL(clicked()),
                            this, SLOT(showDictCopyright()), false,
                            i18n("Copyright"));
    
    createWayToSearchPopupMenu();
}
void KEBook::createStatusBar()
{
    KStatusBar* status = statusBar();
    //status->setBorderWidth(5);
    
    m_pb_lookingup = new KProgress(0, 100, 0,
                                   KProgress::Horizontal, status,
                                   "Looking up progress");
    CHECK_PTR(m_pb_lookingup);
    m_pb_lookingup->setBarStyle(KProgress::Blocked);
    m_pb_lookingup->setTextEnabled(true);
    m_pb_lookingup->setBackgroundColor(status->backgroundColor());
    status->addWidget(m_pb_lookingup, 10);
    
    m_lbl_message = new QLabel(i18n("Unselected Dictionary"), status);
    status->addWidget(m_lbl_message, 100);

    //enableStatusBar();
}
KEBook::KEBook()
    :  KMainWindow::KMainWindow(),
       view(new KEBookWidget(this)),
       m_dictionary_list_popup(new QPopupMenu)
{
    // Tell the KMainWindow that this is indeed the main widget
    setCentralWidget(view);

    
    // Create QAccel object
    m_function_key = new QAccel(view);
    for (int index = 0; Key_F2+index <= Key_F12; index++) {
        m_function_key->insertItem(Key_F2+index, index);
    }
    
    WayToSearchTableElement
        forward_match(this, KEBookApp::FRONT, ID_MENUITEM_SELECT_FORWARD_MATCH,
                      i18n("Forward match"), "Forward match",
                      CTRL+SHIFT+Key_F,
                      MATCH_FORWARD_TBBTN_PIXMAP),
        exact_match(this, KEBookApp::EXACT, ID_MENUITEM_SELECT_EXACT_MATCH,
                    i18n("Exact match"), "Exact match",
                    CTRL+SHIFT+Key_E,
                    MATCH_EXACT_TBBTN_PIXMAP),
        backward_match(this, KEBookApp::TAIL, ID_MENUITEM_SELECT_BACKWARD_MATCH,
                       i18n("Backward match"), "Backward match",
                       CTRL+SHIFT+Key_T,
                       MATCH_BACKWARD_TBBTN_PIXMAP);
    m_way_to_search_table = new vector<WayToSearchTableElement>;
    m_way_to_search_table->push_back(forward_match);
    m_way_to_search_table->push_back(exact_match);
    m_way_to_search_table->push_back(backward_match);
    
    // Create status bar
    createStatusBar();
    
    readConfig(); // Read previous configuration
    
    // Create menu
    createMainMenu();
    // Create tool bar
    createToolBar();
    toolBar()->show();

    connect(getApp(), SIGNAL(sigReleaseSubbook()),
            SLOT(onReleaseSubbook()));
    connect(getApp(), SIGNAL(sigSetDictPath()),
            SLOT(onSetDictPath()));
    connect(getApp(), SIGNAL(sigSetSubbook()),
            SLOT(onSetSubbook()));
    QObject::connect(this, SIGNAL(sigCloseMainWindow()),
                     getApp(), SLOT(quit()));

    // Set caption
    setCaption(PACKAGE);
}

KEBook::~KEBook()
{
    delete m_way_to_search_table;
}
void KEBook::closeEvent(QCloseEvent* ev)
{
    saveConfig();
    emit sigCloseMainWindow();
    KMainWindow::closeEvent(ev);
}
void KEBook::keyPressEvent(QKeyEvent* ev)
{
    if (ev->key() == Key_Escape) {
        getApp()->stopLookingUp();
    }
}
void KEBook::resizeEvent(QResizeEvent* ev)
{
    KMainWindow::resizeEvent(ev);
    m_curgeometry = geometry();
}
void KEBook::moveEvent(QMoveEvent* ev)
{
    KMainWindow::moveEvent(ev);
    m_curgeometry = geometry();
}
void KEBook::saveDictDirHistoryProperties(KConfig* config)
{
    // Get Dictionary Directory History from config
    KEBookApp::DictDirHistory::const_iterator it;
    QString dictdir_history = "";
    char comma = '\0';

    for (it = getApp()->getBeginOfDictDirHistory();
         it != getApp()->getEndOfDictDirHistory();
         it++) {
        dictdir_history = (*it).c_str()+(comma+dictdir_history);
        comma = ':';
    }
    config->setGroup(CFG_PATH_GROUP);
    config->writeEntry(CFG_DICTDIR_HISTORY_ENTRY, dictdir_history);
}
void KEBook::saveProperties(KConfig *config)
{
    config->setGroup(CFG_GEOMETRY_GROUP);
    config->writeEntry(CFG_MAIN_WINDOW_ENTRY, m_curgeometry);
    config->setGroup(CFG_RESULTPART_GROUP);
    config->writeEntry(CFG_BACKGROUNDCOLOR_ENTRY,
                       view->getResultPartBackgroundColor());
    config->writeEntry(CFG_TEXTCOLOR_ENTRY,
                       view->getResultPartTextColor());
    config->writeEntry(CFG_LINKCOLOR_ENTRY,
                       view->getResultPartLinkColor());
    
    saveDictDirHistoryProperties(config);
    
    KMainWindow::saveProperties(config);
}
void KEBook::readSubbookInfoListProperties(KConfig* config)
{
    KEBookApp::SubbookInfoListIterator it;
    it = getApp()->getBeginOfSubbookInfoList();
    
    if (it != getApp()->getEndOfSubbookInfoList()) {
        config->setGroup(CFG_GENERAL_GROUP);
        int i = config->readNumEntry(CFG_SUBBOOK_PREVNUM_ENTRY,
                                     DEFAULT_PREV_SUBBOOK);
        if (i != DEFAULT_PREV_SUBBOOK) {
            QString dir;
            dir = config->readEntry(CFG_SUBBOOK_PREVDIR_ENTRY,
                                    DEFAULT_PREV_SUBBOOK_DIR);
            if (dir == (*(it+i)).directory.c_str()) {
                setSubbook(i);
                if (getApp()->getCurrentError().getErrorCode()
                    != EB_NO_ERR) {
                    cerr << getApp()->getCurrentError().what() << endl;
                    getApp()->setNoError();
                }
            }
        }
    } else if (++it == getApp()->getEndOfSubbookInfoList()){
        setSubbook(0);
        if (getApp()->getCurrentError().getErrorCode()
            != EB_NO_ERR) {
            cerr << getApp()->getCurrentError().what() << endl;
            getApp()->setNoError();
        }
    }
}
void KEBook::readDictDirHistoryProperties(KConfig* config)
{
    config->setGroup(CFG_PATH_GROUP);
    QString dictdir_history = config->readEntry(CFG_DICTDIR_HISTORY_ENTRY, "");
    int p = 0, q = 0;
    const char* dir;
    while ((p = dictdir_history.find(':', q)) != -1) {
        dir = (const char*)dictdir_history.mid(q, p-q).local8Bit();
        getApp()->appendDictDirHistory(dir);
        q = p+1;
    }
    
    dir = (const char*)dictdir_history.right(dictdir_history.length()-q);
    if (dir) {
        getApp()->appendDictDirHistory(dir);
    }
}
void KEBook::readProperties(KConfig *config)
{
    config->setGroup(CFG_RESULTPART_GROUP);
    view->setResultPartBackgroundColor(config->readColorEntry(CFG_BACKGROUNDCOLOR_ENTRY,
                                                              &HTML_DEFAULT_BACKGROUND_COLOR));
    view->setResultPartTextColor(config->readColorEntry(CFG_TEXTCOLOR_ENTRY,
                                                        &HTML_DEFAULT_TEXT_COLOR));
    view->setResultPartLinkColor(config->readColorEntry(CFG_LINKCOLOR_ENTRY,
                                                        &HTML_DEFAULT_LINK_COLOR));
    // Path
    config->setGroup(CFG_PATH_GROUP);
    QString dict_path =
        config->readEntry(CFG_DICT_PATH_ENTRY,
                          DEFAULT_DICT_PATH);
    if (KEBookApp::isAvailableDictPath(dict_path)) {
        getApp()->setDictPath(dict_path);
    } else {
        return ;
    }
    
    readSubbookInfoListProperties(config);

    readDictDirHistoryProperties(config);
    
    KMainWindow::readProperties(config);
}
void KEBook::setSubbook(int i)
{
    getApp()->setSubbookWithIndex(i);
    
    showDictCopyright();
    setCaption(QString::fromLocal8Bit(getApp()->getSubbookTitle()));
    view->clearMatchWordsList();
    m_lbl_message->setText(i18n("Ready"));
    emit sigSelectedSubbook();
}
void KEBook::setSubbook(const KEBookApp::DictSpec& dictspec)
{
    getApp()->setDictionary(dictspec);
    
    showDictCopyright();
    setCaption(QString::fromLocal8Bit(getApp()->getSubbookTitle()));
    view->clearMatchWordsList();
    m_lbl_message->setText(i18n("Ready"));
    emit sigSelectedSubbook();
}
void KEBook::openSelectWayToSearchDlg()
{
    SelectWayToSearchDlg* dlg =
        new SelectWayToSearchDlg(getApp()->getWayToSearch(),
                                 this, "SelectWayToSeachDlg");
    if (dlg->exec()) {
        getApp()->setWayToSearch(dlg->getWayToSearch());
    }
    delete dlg;
}
void KEBook::openSelectSubbookDlg()
{
    vector<string> list;
    getApp()->getSubbookTitleList(list);

    int selected_subbook = getApp()->getSubbook();
    SelectSubbookDlg* dlg = new SelectSubbookDlg(list, selected_subbook, this);

    if (dlg->exec() &&
        dlg->getSelectedSubbook() != selected_subbook) {
        setSubbook(dlg->getSelectedSubbook());
    }
    delete dlg;
}
bool KEBook::setDictPath(const QString& path)
{
    if (!path.isEmpty()) {
        if (KEBookApp::isAvailableDictPath(path)) {
            getApp()->setDictPath(path);
            menuBar()->setItemEnabled(ID_MENUITEM_OPEN_SELECT_DICT_DLG, true);
        } else {
            KMessageBox::error((QWidget *)this, PACKAGE "--Error",
                               i18n("The directory which you select now "
                                    "seems not to contain the dictionaries."));
            return false;
        }
        KEBookApp::SubbookInfoListIterator it;
        it = getApp()->getBeginOfSubbookInfoList();
        if (++it == getApp()->getEndOfSubbookInfoList()) {
            setSubbook(0);
        } else {
            getApp()->unsetSubbook();
            openSelectSubbookDlg();
            if (getApp()->getSubbook() == ebook::RC_ERR) {
                setCaption(i18n("No dictionary is selected."));
                view->printResult("No dictionary is selected.",
                                  (const char*)i18n("No Dictionary is selected.").local8Bit());
            }
        }
        onSetDictPath();
    }
    return true;
}
void KEBook::openDictPathDlg(const char* dict_path)
{
    QString new_dict_path =
        KFileDialog::getExistingDirectory(dict_path, this,
                                          i18n("Select a directory to contain the dictionary."));
    if (new_dict_path != QString::null) {
        if (!setDictPath(new_dict_path)) {
            openDictPathDlg(dict_path);
        } else {
            updateDictDirHistoryMenu();
        }
    }
}
void KEBook::preLookupWord(const char* word)
{
    emit sigSuspendGUIParts();
    toolBar()->setItemEnabled(ID_TBBTN_STOPLOOKINGUP, true);
    string message((const char*)i18n("Now Looking up "));
    message += word;
    message += i18n("...");
    m_lbl_message->setText(QString::fromLocal8Bit(message.c_str()));
}
void KEBook::postLookupWord()
{
    m_pb_lookingup->setValue(0);
    m_pb_lookingup->update();
    m_lbl_message->setText(i18n("Ready"));
    emit sigResumeGUIParts();
    toolBar()->setItemEnabled(ID_TBBTN_STOPLOOKINGUP, false);
}
void KEBook::showDictMenu()
{
    getApp()->clearCurResult();
    string menu;
    getApp()->getDictMenu(menu);
    view->printResult("Menu", menu);
}
void KEBook::onReleaseSubbook()
{
    m_lbl_message->setText(i18n("Unselected Dictionary"));
}
void KEBook::onSetSubbook()
{
    menuBar()->setItemEnabled(ID_MENUITEM_COPYRIGHT, true);
    toolBar()->setItemEnabled(ID_TBBTN_COPYRIGHT, true);
    menuBar()->setItemEnabled(ID_MENUITEM_DICT_MENU, true);
    toolBar()->setItemEnabled(ID_TBBTN_MENU, true);
}
void KEBook::onSetDictPath()
{
    menuBar()->setItemEnabled(ID_MENUITEM_OPEN_SELECT_DICT_DLG, true);
    toolBar()->setItemEnabled(ID_TBBTN_SETSUBBOOK, true);
}
void KEBook::showDictCopyright()
{
    getApp()->clearCurResult();
    string copyright;
    getApp()->getCopyright(copyright);
    view->printResult("Copyright", copyright);
}
bool KEBook::event(QEvent* ev)
{
    switch (ev->type()) {
    case QEvent::Create:
        return false;
        
    case QEvent::Show:
        {
            // Geometry
            KConfig* config = kapp->config();
            config->setGroup(CFG_GEOMETRY_GROUP);
            const QRect default_rect(0, 35, 600, 635);
            setGeometry(config->readRectEntry(CFG_MAIN_WINDOW_ENTRY,
                                              &default_rect));
            return false;
        }
    case Event_LookupFinish:
        view->finishLookup();
        postLookupWord();
        break;
    case Event_Progress:
        {
            KEBookProgressEvent* prog_ev
                = (KEBookProgressEvent*)ev;
            m_pb_lookingup->setValue(prog_ev->getPercent());
            emit sigAddMatchWordsList(prog_ev->getIndex(),
                                      prog_ev->getCount());
            break;
        }
    default:
        KMainWindow::event(ev);
        break;
    }
    return false;
}
void KEBook::tellWayToSearchToEngine(int menu_id)
{
    switch (menu_id) {
    case ID_MENUITEM_SELECT_FORWARD_MATCH:
        getApp()->setWayToSearch(KEBookApp::FRONT);
        break;
    case ID_MENUITEM_SELECT_EXACT_MATCH:
        getApp()->setWayToSearch(KEBookApp::EXACT);
        break;
    case ID_MENUITEM_SELECT_BACKWARD_MATCH:
        getApp()->setWayToSearch(KEBookApp::TAIL);
        break;
    }
    updateSelectionOfWayToSearch();
}
void KEBook::updateSelectionOfWayToSearch()
{
    KEBookApp::WAY_TO_SEARCH way_to_search = getApp()->getWayToSearch();
    QPixmap icon;
    //QString kebook_toolbar_dir;
    //getKEBookToolbarDir(kebook_toolbar_dir);
    icon = BarIcon(WAY_TO_SEARCH_TBBTN_PIXMAP);
    vector<WayToSearchTableElement>::iterator it;
    it = m_way_to_search_table->begin();
    while (it != m_way_to_search_table->end()) {
        if (way_to_search == it->getWayToSearch()) {
            menuBar()->setItemChecked(it->getMenuId(), TRUE);
            icon = BarIcon(it->getIconFile());
        } else {
            menuBar()->setItemChecked(it->getMenuId(), FALSE);
        }
        it++;
    }
    toolBar()->setButtonPixmap(ID_TBBTN_WAY_TO_SEARCH, icon);
}
bool KEBook::rescanDictionariesInCDROM()
{
    QPixmap icon;
    //QString common_toolbar_dir;
    //getCommonToolbarDir(common_toolbar_dir);
    icon = BarIcon(SETSUBBOOK_TBBTN_PIXMAP);
    m_dictionary_list_popup->clear();
    if (!getApp()->rescanDictSpecListInCDROM()) {
        return false;
    }
    KEBookApp::DictSpecListIterator it;
    it = getApp()->getBeginOfDictSpecListInCDROM();
    if (it != getApp()->getEndOfDictSpecListInCDROM()) {
        int id = 0;
        while (it != getApp()->getEndOfDictSpecListInCDROM()) {
            QString caption = (*it++).title.c_str()
                +('\t'+KAccel::keyToString(Key_F2+id));
            m_dictionary_list_popup->insertItem(icon,
                                                QString::fromLocal8Bit(caption), id);
            id++;
        }
        m_dictionary_list_popup->setEnabled(true);
    } else {
        m_dictionary_list_popup->setEnabled(false);
    }
    return true;
}
void KEBook::onSelectDictionaryFromList(int i)
{
    if (getApp()->getBeginOfDictSpecListInCDROM()+i
            >= getApp()->getEndOfDictSpecListInCDROM()) {
        return;
    }
    KEBookApp::DictSpec spec = *(getApp()->getBeginOfDictSpecListInCDROM()+i);
    if (KEBookApp::isAvailableDictPath(spec.directory.c_str())) {
        setSubbook(spec);
    } else {
        KMessageBox::error(this, PACKAGE "--Error",
                           i18n("Error occurred to me on changing dictionary.\n"
                                "Please check if your CD-ROM is mounted and "
                                "[Rescan dictionaries] from [CD-ROM] menu.\n"));
    }
}
void KEBook::onSelectDictDirFromList(int menu_id)
{
    KEBookApp::DictDirHistory::const_iterator it;

    for (it = getApp()->getBeginOfDictDirHistory();
         it != getApp()->getEndOfDictDirHistory();
         it++, menu_id--) {
        if (menu_id == 0) {
            setDictPath(QString::fromLocal8Bit((*it).c_str()));
            return;
        }
    }
    KMessageBox::error((QWidget *)this, PACKAGE "--Error",
                       i18n("The directory which you select now "
                            "seems not to contain the dictionaries."));
}
#include "kebook.moc"
