/*
 * $Id: resultview.cpp,v 1.8 2001/01/31 07:13:19 hige Exp $
 *
 * Copyright (C) 1999 SUZUKI, Mitsuhiro <hige@higesoft.rim.or.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <kapp.h>
#include <khtml_part.h>
#include <khtml_events.h>

#include <qwidget.h>
#include <qclipboard.h>
#include <qkeycode.h>

#include <strstream>
#include <iomanip>

#include "kebookwidget.h"
#include "htmlcolors.h"
#include "resultview.h"

#define  HTML_TABLE_START               "<table cellspacing=\"0\" cellpadding=\"0\"><tr>"
#define  HTML_TABLE_END                 "</tr></table>"
#define  HTML_TABLE_DATA_START          "<td nowrap>"
#define  HTML_TABLE_DATA_END            "</td>"
#define  HTML_IMG                       "<img "

#define  EMPTY  0

static const int WRAP_DELAY = 10;


template <class T>
inline T max(T x, T y)
{
    return (x > y) ? x : y;
}
KResultPart::KResultPart(int fontsize,
                         QWidget *_parent, const char *_name)
    : KHTMLPart::KHTMLPart(_parent, _name),
      m_margin(WRAP_DELAY*fontsize),
      m_offset_vert(0),
      m_fontsize(fontsize)
{
    QValueList<int> fontsizes;
    for (int i = 0; i < 7; i++)
        fontsizes.append(m_fontsize);
    setFontSizes(fontsizes);

    m_colorattr.background = HTML_DEFAULT_BACKGROUND_COLOR;
    m_colorattr.text = HTML_DEFAULT_TEXT_COLOR;
    m_colorattr.link = HTML_DEFAULT_LINK_COLOR;
    m_colorattr.vlink = HTML_DEFAULT_LINK_COLOR;
    
    setSelectable(true);
    setEncoding(QString("euc-jp"));
}
KResultPart::~KResultPart()
{
}
void KResultPart::slotScrollVert(int y)
{
    m_offset_vert = max(y, 0);
    view()->scrollBy(0, m_offset_vert);
}
void KResultPart::slotScrollVertUp()
{
    view()->scrollBy(0, -m_fontsize);
}
void KResultPart::slotScrollVertDown()
{
    view()->scrollBy(0, m_fontsize);
}
void KResultPart::slotPageUp()
{
    QKeyEvent* ev = new QKeyEvent(QEvent::KeyPress, Key_PageUp, 0, 0);
    KApplication::postEvent(this, ev);
}
void KResultPart::slotPageDown()
{
    QKeyEvent* ev = new QKeyEvent(QEvent::KeyPress, Key_PageDown, 0, 0);
    KApplication::postEvent(this, ev);
}
void wrapText(string& wraptext,
              const char* nowraptext,
              int col,
              const char* row_start,
              const char* row_end)
{
    const size_t maxlen = strlen(nowraptext)+1;
    int nowcol = col;
    bool hyphen = false;
    
    while (*nowraptext != '\0') {
        if (*nowraptext == '<') {
            if (strncmp(nowraptext+1, "br", strlen("br")) == 0
                || (*(nowraptext+1) == 'p' && !isalpha(*(nowraptext+2)))
                || strncmp(nowraptext+1, "table", strlen("table")) == 0) {
                nowcol = col;
            }
            if (strncmp(nowraptext+1, "img", strlen("img")) == 0) {
                nowcol -= 2;
            }
            do {
                wraptext += *nowraptext;
            } while (*nowraptext++ != '>');
        } else {
            int m = mblen(nowraptext, maxlen);
            if (m == 1 && isalpha(*nowraptext) &&
                mblen(nowraptext+m, maxlen) == 1 &&
                isalpha(*(nowraptext+m))) {
                hyphen = true;
            } else {
                hyphen = false;
            }
            nowcol -= m;
            while (m-- > 0) {
                wraptext += *nowraptext++;
            }
        }
        if ((!hyphen && nowcol < 0) || (hyphen && nowcol+WRAP_DELAY < 0)) {
            if (hyphen) {
                wraptext += '-';
                hyphen = false;
            }
            wraptext += row_end;
            wraptext += '\n';
            wraptext += row_start;
            nowcol = col;
        }
    }
}
void KResultPart::urlSelected(const QString &url,
                              int button, int /* state */,
                              const QString& /* _target*/)
{
    if (url != "") {
        switch (button) {
        case LeftButton:
            KURL u(url);
            if (strcmp(u.protocol(), "eb") == 0) {
                long l = strtol(u.host(), EMPTY, 10);
                
                ClickLinkEvent* ev = new ClickLinkEvent((int)l, u.port(), button);
                KApplication::postEvent(this, ev);
            }
            break;
        }
    }
}
void KResultPart::khtmlMouseReleaseEvent(khtml::MouseReleaseEvent *event)
{
    KHTMLPart::khtmlMouseReleaseEvent(event);
    switch (event->qmouseEvent()->button()) {
    case LeftButton:{
        if (hasSelection()) {
            QString selectstr = selectedText();
            if (selectstr) {
                QClipboard* cb = KApplication::clipboard();
                cb->setText(selectstr);
            }
        }
        break;
    }
    default:
        break;
        
    }
    return;
}
void KResultPart::printContent(const string& title, const string& content,
                               const char* row_start,
                               const char* row_end)
{
    // Text wrap
    int col = (2*(view()->width()-m_margin))/m_fontsize;
    string wrapped_text;
    wrapText(wrapped_text, content.c_str(), col, row_start, row_end);
    
    const QColor& bgcolor = m_colorattr.background;
    const QColor& text = m_colorattr.text;
    const QColor& link = m_colorattr.link;
    const QColor& vlink = m_colorattr.vlink;

    strstream html;
    html << "<html>" << endl
         << "<head>" << endl
         << "<title>" << title << "</title>" << endl
         << "</head>" << endl
         << "<body bgcolor=\"#"
         << hex << setfill('0')
         << setw(2) << bgcolor.red()
         << setw(2) << bgcolor.green()
         << setw(2) << bgcolor.blue()
         << "\" text=\"#"
         << setw(2) << text.red()
         << setw(2) << text.green()
         << setw(2) << text.blue()
         << "\" link=\"#"
         << setw(2) << link.red()
         << setw(2) << link.green()
         << setw(2) << link.blue()
         << "\" vlink=\"#" << setw(2)
         << setw(2) << vlink.red()
         << setw(2) << vlink.green()
         << setw(2) << vlink.blue()
         << "\" >"
         << endl
         << wrapped_text
         << "</body>" << endl
         << "</html>" << ends;
    begin();
    write(QString::fromLocal8Bit(html.str()));
    end();
}
bool KResultPart::event(QEvent* ev)
{
    switch (ev->type()) {
    case ClickLinkEvent::ClickLink: {
        ClickLinkEvent* click_ev = (ClickLinkEvent*)ev;
        emit sigClickLink(click_ev->frame(), click_ev->offset());
        return false;
    }
    default:
        return KHTMLPart::event(ev);
    }
}

#include "resultview.moc"
