/*
 * $Id: resultview.h,v 1.2 2000/09/23 00:28:07 hige Exp $
 *
 * Copyright (C) 1999 SUZUKI, Mitsuhiro <hige@higesoft.rim.or.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
#ifndef KDEBOOK_RESULTVIEW_HPP
#define KDEBOOK_RESULTVIEW_HPP

#include <qevent.h>

#include <string>
#include <khtmlview.h>
#include <khtml_part.h>

class QPoint;

class ClickLinkEvent : public QEvent
{
public:
    static const Type ClickLink = (Type)(QEvent::User+5);
    
    ClickLinkEvent(int frame, long offset, int button)
        : QEvent(ClickLink),
        m_frame(frame),
        m_offset(offset),
        m_button(button) {}

    const int frame() const {return m_frame;}
    const long offset() const {return m_offset;}
    const int button() const {return m_button;}

    
private:
    int m_frame;
    long m_offset;
    int m_button;
};

class KResultPart : public KHTMLPart
{
    Q_OBJECT
public:
    //======== Constructor/Destructor ========//
    KResultPart(int fontsize,
                QWidget* _parent = 0L, const char* _name = 0L);
    virtual ~KResultPart();

    void printContent(const string& title, const string& content,
                      const char* row_start,
                      const char* row_end);
    int offsetVertical() {return m_offset_vert;}

    virtual bool event(QEvent* ev);
    
    //========= Color ===========//
    struct ColorAttribute {
        QColor background, text, link, vlink;
    };
    const ColorAttribute& getColorAttribute() {return m_colorattr;}
    void setColorAttribute(const ColorAttribute& colorattr)
        {m_colorattr = colorattr; emit sigChangeColorAttribute();}
    
    void setBackgroundColor(const QColor& newcolor)
        {m_colorattr.background = newcolor;}
    void setTextColor(const QColor& newcolor)
        {m_colorattr.text = newcolor;}
    void setLinkColor(const QColor& newcolor)
        {m_colorattr.link = newcolor;}
signals:
    void sigClickLink(int page, long offset);
    void sigChangeColorAttribute();

public slots:
    void slotScrollVert(int y);
    void slotScrollVertUp();
    void slotScrollVertDown();
    void slotPageUp();
    void slotPageDown();
    
protected:
    virtual void urlSelected(const QString& url,
                             int button = 0, int state = 0,
                             const QString& _target = QString::null);
    virtual void khtmlMouseReleaseEvent(khtml::MouseReleaseEvent* event);
    
 private:
    int m_wrapcol;
    int m_margin;
    int m_offset_vert;
    int m_fontsize;
    ColorAttribute m_colorattr;
};

#endif /* KEBOOK_RESULTVIEW */
